/***************************************************************************
 *  Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "paper_xml.h"
#include <QApplication>
#include <QDir>


IArray * iarrayFromFile (QString path, IOpenGLHelper * helper, int flags);

void CQXmlHandler_Paper::clear (void) {
    foreach (sPaper * p, cur_list) {
        SAFE_RELEASE (p->height);
        SAFE_RELEASE (p->color);
        for (int i = 0; i < p->filtered_count; i++) {
            SAFE_RELEASE (p->filtered[i].filtered);
        }

        delete p;
    }

    cur_list.clear ();
    open = FALSE;
}

CQXmlHandler_Paper::CQXmlHandler_Paper (IOpenGLHelper * helper) {
    format    = 0;
    open      = FALSE;
    cur_paper = NULL;
    m_Helper  = helper;
}

CQXmlHandler_Paper::~CQXmlHandler_Paper (void) {
    clear ();
}

bool CQXmlHandler_Paper::startElement (
                                const QString &,
                                const QString &,
                                const QString & name,
                                const QXmlAttributes & attributes) {

     if (name == "format") {
         // Format descriptor.
         //
         bool good;
         INT type = attributes.value ("type").toInt (&good, 10);
         if (good)
             format = type;
     } else if (name == "pack") {
         // Paper pack begins here
         //
         open = TRUE;
     } else if (name == "paper") {
         if (open) {
             SAFE_DELETE (cur_paper);
             cur_paper = new sPaper ();

             // Set the default values.
             cur_paper->color  = NULL;
             cur_paper->height = NULL;
             cur_paper->dpi    = 100;
             cur_paper->name   = "";
             cur_paper->paper_class = sPaper::PLAIN;
             cur_paper->filtered_count = 0;
             for (int i = 0; i < PREFILTERED_PAPER_COUNT; i++) {
                 cur_paper->filtered[i].filtered = NULL;
                 cur_paper->filtered[i].filter_r = 0.0f;
             }

             // Paper declaration begins here
             //
             QString name = attributes.value ("name");
             if (!name.isEmpty ()) {
                 cur_paper->name = name;
             }

             QDir dir = QDir (DATA_PATH);
             dir.cd ("images");
             dir.cd ("paper");

             QString color = attributes.value ("color");
             if (!color.isEmpty ()) {
                 cur_paper->color = iarrayFromFile (dir.absoluteFilePath (color),
                                                    m_Helper, IArray::FLAGS_NO_SHARE);
             }

             QString height = attributes.value ("height");
             if (!height.isEmpty ()) {
                 cur_paper->height = iarrayFromFile (dir.absoluteFilePath (height),
                                                     m_Helper, IArray::FLAGS_NO_SHARE);
             }

             QString pclass = attributes.value ("pclass");
             if (!pclass.isEmpty ()) {
                 bool ok = FALSE;
                 cur_paper->paper_class = (sPaper::ePaperClass) pclass.toInt (&ok, 10);
                 if (!ok) {
                     cur_paper->paper_class = sPaper::PLAIN;
                 }
             }

             QString dpi = attributes.value ("dpi");
             if (!pclass.isEmpty ()) {
                 bool ok = FALSE;
                 cur_paper->dpi = (sPaper::ePaperClass) dpi.toInt (&ok, 10);
                 if (!ok) {
                     cur_paper->dpi = 100;
                 }
             }
         }
     } else if (name == "filtered") {
         if (cur_paper) {
             if (cur_paper->filtered_count < PREFILTERED_PAPER_COUNT) {
                 const int idx = cur_paper->filtered_count;

                 QString r = attributes.value ("r");
                 if (!r.isEmpty ()) {
                     bool ok = FALSE;
                     cur_paper->filtered[idx].filter_r = r.toInt (&ok, 10);
                     if (!ok) {
                         return TRUE;
                     }
                 }

                 QDir dir = QDir (DATA_PATH);
                 dir.cd ("images");
                 dir.cd ("paper");

                 QString path = attributes.value ("path");
                 if (!path.isEmpty ()) {
                     IArray * filtered = iarrayFromFile (dir.absoluteFilePath (path), m_Helper,
                                                         IArray::FLAGS_NO_SHARE);
                     if (!filtered) {
                         return TRUE;
                     }

                     cur_paper->filtered[idx].filtered = filtered;
                 }

                 cur_paper->filtered_count++;
             }
         }
     }

    return TRUE;
}

bool CQXmlHandler_Paper::endElement (
                                const QString &,
                                const QString &,
                                const QString & name) {

    if (name == "paper") {
        // Paper declaration ends here
        //
        if (cur_paper) {
            if (cur_paper->height)
                cur_list.push_front (cur_paper);
            else {
                // Incomplete description.
                // Rollback.
                SAFE_RELEASE (cur_paper->color);
                SAFE_RELEASE (cur_paper->height);
                for (int i = 0; i < cur_paper->filtered_count; i++) {
                    SAFE_RELEASE (cur_paper->filtered[i].filtered);
                }

                delete cur_paper;
            }
        }

        cur_paper = NULL;
    } else if (name == "pack") {
        // Pack is finished.
        //
        SAFE_DELETE (cur_paper);
        open = FALSE;
    }

    return TRUE;
}

bool CQXmlHandler_Paper::characters (const QString &) {
    // O.o
    return TRUE;
}

bool CQXmlHandler_Paper::fatalError (const QXmlParseException &) {
    // O.o
    return TRUE;
}

QString CQXmlHandler_Paper::errorString (void) const {
    return "o.O ... I know nothing.";
}
