#ifndef PALETTE_H
#define PALETTE_H

#include "iPalette.h"
#include "iInstrument.h"
#include "pigmentbox.h"
#include <QGridLayout>

#include "ui_palette.h"


class Palette : public QFrame,
                public IPalette,
                public CRefCounter {
    Q_OBJECT

public:

    virtual QWidget * asWidget (void) {
        return this;
    }

    /**
     * Set the palette orientation.
     */
    virtual void setVertical (bool v) {
        m_Orientation = !v;
    }

    /**
     * Get the palette orientation.
     */
    virtual bool vertical (void) {
        return m_Orientation;
    }

    /**
     * Ask the palette window to hide
     * itself if it doesn't mind.
     */
    virtual void askToHide (void) {
        if (m_Hidable)
            hide ();
    }

    /**
     * Set the columns count if the palette is
     * horizontal, or the rows count if it
     * is vertical.
     */
    virtual void setRowsCount (int len) {
        m_Length = len;
    }

    /**
     * Get the row count.
     */
    virtual int rowsCount (void) {
        return m_Length;
    }

    /**
     * Add a pigment to palette.
     */
    virtual void addPigment (IPigmentMixture * pigment);

    /**
     * Remove all pigments from
     * the palette.
     */
    virtual void clear (void);

    /**
     * Load the palette from file.
     */
    virtual bool load (const char * path);

    /**
     * Save the palette to file.
     */
    virtual bool save (const char * path);

    /**
     * Sort the colors in palette.
     */
    virtual void sort (ePaletteSortIndex index, bool asc);

    /********* ICONNECTORCALLBACK IMPLANTS ***********/

    /**
     * Pass the connector interface
     * to the object.
     */
    virtual void connect (IConnector * caller);

    /**
     * A callback, called by connector when
     * the requested data is recieved.
     * @param param Caller-define parameter associated with request.
     * @param length The length of the data string in bytes.
     * @param data Data returned. This buffer will be invalid after the function exit.
     */
    virtual void complitedRequest (int param, int length, void * data);

    /*********** ISPAWNEDOBJECT IMPLANTS *************/

    /**
     * Get the classname for this object.
     */
    virtual const char * myClassName (void) {
        return "Client::Palette";
    }

    /**
     * Get the Id of current allocated object.
     */
    virtual int objectId (void) {
        return m_Id;
    }

    /**
     * Increase the object's
     * reference counter.
     */
    virtual void addRef (void) {
        CRefCounter::addRef ();
    }

    /**
     * Release the object.
     */
    virtual void release (void) {
        CRefCounter::release ();
    }

    /**
     * Ask the object to emit all
     * notification signals if any.
     */
    virtual void pokeObject (void) {
    }

signals:
    /**
     * Emitted each time the user touches
     * any pigment.
     */
    void touched (IPigmentMixture * pigment, float amount, eColorMetod source);

protected:
    void rebound (void);
    virtual void resizeEvent (QResizeEvent * event );
    virtual void showEvent   (QShowEvent * event);
	
private slots:
    void poke        (IPigmentMixture * pig, float amount);
    void editPigment (PigmentBox * box);
    void insertCopyOfPigment (IPigmentMixture * pigment);

  //void flip        (void);

public slots:
    void sortingChanged (int idx);
    void savePalette    (void);
    void loadPalette    (void);

friend IPalette * g_fac_SPAWN_PALETTE (QWidget * parent, ISkin * skin, bool isstatic);
friend IPalette * g_fac_SPAWN_PALETTE (void);
private:
    Ui::Palette     ui;

    int             m_Id;
    int             m_Length;
    int             m_MaxDynColors;
    bool            m_Orientation;
    bool            m_Hidable;
    bool            m_IsDynamic;
    eColorPaintMode m_PaintMode;

    IConnector            * m_Connector;
    ISkin                 * m_Skin;

    QList<PigmentBox*>      m_Colours;
    QGridLayout           * m_Grid;

    Palette (QWidget * parent, ISkin * skin, bool isstatic);
    virtual ~Palette (void);

    virtual void suicide (void) {
        delete this;
    }
};

#endif // PALETTE_H
