#include "pigmentbox.h"
#include "qhasmath.h"
#include <QPainter>
#include <QTabletEvent>

PigmentBox::PigmentBox (QWidget * parent, ISkin * skin) : QFrame (parent) {
    m_Pigment     = NULL;
    m_Skin        = skin;
    m_PaintMode   = PAINT_KM;
    m_Orientation = FALSE;
    m_Dragging    = FALSE;
    if (skin) {
        skin->addRef ();
    }

    setMinimumSize  (16, 16);
    setMaximumSize  (16, 16);
}

PigmentBox::~PigmentBox (void) {
    SAFE_RELEASE (m_Pigment);
    SAFE_RELEASE (m_Skin);
}

void PigmentBox::setPigment (IPigmentMixture * pig) {
    SAFE_RELEASE (m_Pigment);

    m_Pigment = pig;
    if (pig) {
        pig->addRef ();
        setToolTip (tr (pig->name ()));
    }
}

void PigmentBox::paintEvent (QPaintEvent * e) {
    Q_UNUSED (e);
    QPainter painter (this);

    QRect rInner = rect ().adjusted (1, 1, -1, -1);

    if (m_Pigment) {
        float rgba[4];
        KubelkaMunk::colorOverWhite (m_Pigment->absorbing (), m_Pigment->scattering (), rgba);
        painter.setBrush (QBrush (QColor (0, 0, 0)));
        painter.drawRect (rect ());
        painter.fillRect (rInner, QBrush (QColor (
                (int)(255.0f * rgba[0]),
                (int)(255.0f * rgba[1]),
                (int)(255.0f * rgba[2]),
                255
        )));
    } else {
        painter.setBrush (QBrush (QColor (0, 0, 0)));
        painter.drawRect (rect ());
        painter.fillRect (rInner, QBrush (QColor (200, 200, 200)));
    }
}

void PigmentBox::mousePressEvent (QMouseEvent * e) {
    if (Qt::LeftButton == e->button ()) {
        m_Dragging = TRUE;
        emit touched (m_Pigment, 1.0f);
    }
}

void PigmentBox::mouseMoveEvent (QMouseEvent * e) {
    Q_UNUSED (e);
    if (m_Dragging &&
        m_Pigment) {
            emit touched (m_Pigment, 0.05f);
    }
}

void PigmentBox::mouseReleaseEvent (QMouseEvent * e) {
    if (Qt::RightButton == e->button ()) {
        // Open an editing dialog on
        // right click.
        emit editPigmentRequest (this);
    } else
    if (Qt::LeftButton == e->button ()) {
        m_Dragging = FALSE;
    }

    e->accept ();
}

void PigmentBox::leaveEvent (QEvent * event) {
    Q_UNUSED (event);
    m_Dragging = FALSE;
}

void PigmentBox::tabletEvent (QTabletEvent * event) {
    if (IS_SIGNIFICANT (event->pressure ())) {
        if (m_Pigment)
            emit touched (m_Pigment, 1.0f);// * event->pressure ());

        event->accept ();
        return;
    }

    event->accept ();
}
