/* Qaquarelle
 * Copyright (C) 2009 Vasiliy Makarov <drmoriarty.0@gmail.com>
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "newpainting.h"
#include <QObject>

static const int g_nTEMPLATES = 3;
static const PaintingSettings g_TEMPLATES[3] = {
    PaintingSettings (800,  600,  100, 0, QObject::tr ("800 x 600")),
    PaintingSettings (1024, 768,  100, 0, QObject::tr ("1024 x 768")),
    PaintingSettings (1600, 1200, 100, 0, QObject::tr ("1600 x 1200"))
};

NewPainting::NewPainting(PaintingSettings *set, QWidget *parent) : QDialog(parent)
{
	ui.setupUi(this);
        settings = set;
        if(settings->width) {
                ui.spinBox_width->setValue(settings->width);
        }
        if(settings->height) {
                ui.spinBox_height->setValue(settings->height);
        }
        if(settings->sizeType) {
                ui.comboBox_sizeType->setCurrentIndex (settings->sizeType);
        }

        // Refill the combo with predefined settings.
        ui.comboBox_template->clear ();
        for (int i = 0; i < g_nTEMPLATES; i++) {
            ui.comboBox_template->addItem (g_TEMPLATES[i].tmplt_name);
        }

        ui.comboBox_technique->addItem (tr ("Watercolors"));
        ui.comboBox_technique->addItem (tr ("Oils"));
        ui.comboBox_technique->addItem (tr ("Sketch"));
        ui.comboBox_technique->setCurrentIndex (0);

	connect(this, SIGNAL(accepted()), this, SLOT(storeSettings()));
}

void NewPainting::setTemplate(int t)
{
    if (t < g_nTEMPLATES) {
        ui.spinBox_width->setValue  (g_TEMPLATES[t].width);
        ui.spinBox_height->setValue (g_TEMPLATES[t].height);
    }
}

void NewPainting::storeSettings()
{
    if (settings) {
        settings->width     = ui.spinBox_width->value();
        settings->height    = ui.spinBox_height->value();
        settings->dpi       = ui.spinBox_dpi->value();
        settings->sizeType  = ui.comboBox_sizeType->currentIndex();
        settings->technique = ui.comboBox_technique->currentIndex();
    }
}
