#include "work_layerpreview.h"

Work_LayerPreview::Work_LayerPreview (ICanvas * canvas,
                                      ICanvasLayer  * layer,
                                      IOpenGLHelper * helper,
                                      int  priority,
                                      bool persistent,
                                      const sWorkerParameters * params) {
    m_Canvas     = canvas;
    m_Layer      = layer;
    m_Helper     = helper;
    m_Priority   = priority;
    m_Persistent = persistent;
    m_Connector  = NULL;
    m_Logger     = NULL;
    m_Preview    = NULL;
    m_Id         = ISpawnedObject::allocateId ();
    m_Active     = 1;
    m_Params     = params;

    ISpawnedObject::_register (this);

    m_LastError      = NULL;
    m_LoggingEnabled = FALSE;

    if (canvas) canvas->addRef ();
    if (layer)  layer->addRef ();
    if (helper) helper->addRef ();

    // Create a target texture.
    m_Preview = g_fac_SPAWN_ARRAY (m_Helper, PREVIEW_WIDTH, PREVIEW_HEIGHT,
                                   m_Helper->proposeTextureFormat (OPENGLHELPER_TEXTURE_RGBA),
                                   GL_UNSIGNED_BYTE, IArray::FLAGS_NULL);

    m_Previewi = new QImage (PREVIEW_WIDTH, PREVIEW_HEIGHT, QImage::Format_ARGB32);
}

Work_LayerPreview::~Work_LayerPreview (void) {
    ISpawnedObject::_unregister (this);

    logMessage (LOGIMPORTANCE_NEVERMIND, "Work complited -- layer preview.");

    SAFE_DELETE  (m_Previewi);
    SAFE_RELEASE (m_Preview);
    SAFE_RELEASE (m_Canvas);
    SAFE_RELEASE (m_Layer);
    SAFE_RELEASE (m_Helper);
    SAFE_RELEASE (m_Connector);
    SAFE_RELEASE (m_Logger);
}

void Work_LayerPreview::connect (IConnector * caller) {
    SAFE_RELEASE (m_Connector);
    if (caller) {
        m_Connector = caller;
        m_Connector->addRef ();

        // Request a rendering chain.
      //m_Connector->allocateObject (this, 13, ENTITY_RENDERING_CHAIN);
    }
}

void Work_LayerPreview::complitedRequest (int param, int length, void * data) {
    Q_UNUSED (param);
    Q_UNUSED (length);
    Q_UNUSED (data);
}

bool Work_LayerPreview::execute (void) {
    m_LastError = NULL;

    if (m_Active == 0)
        return FALSE;

    if (m_Layer &&
        m_Canvas &&
        m_Preview &&
        m_Previewi &&
        m_Helper) {

        // Render on a backbuffer.
        if (m_Canvas->asImage (m_Preview)) {
            // Convert to QImage.
            // FIXME Detach to a separate method. Optimize.
            const BYTE * srcl = reinterpret_cast<const BYTE*>(m_Preview->lock_pointer_const ());
            for (int y = 0; y < PREVIEW_HEIGHT; y++) {
                QRgb * line = (QRgb*) m_Previewi->scanLine (y);
                const BYTE * src = srcl;
                for (int x = 0; x < PREVIEW_WIDTH; x++) {
                    line[x] = qRgb (src[0], src[1], src[2]);
                    src += 4;
                }

                srcl += PREVIEW_WIDTH * 4;
            }

            m_Preview->unlock ();

            // And broadcast the result.
            emit previewUpdated (m_Previewi);
            return TRUE;
        }
    } else {
        m_LastError = "No preview parameters set up, staying idle.";
    }

    return FALSE;
}
