/***************************************************************************
 *  Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef WORKER_H
#define WORKER_H

#include "commtypes.h"
#include "commdefs.h"
#include "refcounter.h"

//#ifdef QT_OPENGL_LIB
#   include "../../glew/include/GL/glew.h"

#include "iConnector.h"
#include "iCanvasLayer.h"
#include "iCanvas.h"
#include "iOpenGLHelper.h"
#include "iRenderingChain.h"

#include <QObject>
#include <QTimer>
#include <QTime>

#include "iwork.h"
#include "work_layerpreview.h"
#include "work_canvasiterate.h"
#include "worker_params.h"


#define WORKER_INTERVAL         50
#define WORKER_IDLE_WAIT        1024            // Number of ms before becoming idle.
#define WORKER_RESCAN_WAIT      10 * 1024       // Number of ms before rescanning.


struct sTaskPack {
    IWork * work;
    QTime   timer;

    sTaskPack (IWork * w) {
        work = w;
        timer.start ();
    }
};

/**
 * Background tasks manager.
 */
class Worker : public QObject {
    Q_OBJECT

    QList<sTaskPack *> m_Tasks;

    QTime  m_IdleFrom;
    QTime  m_ScanningFrom;
    QTimer m_Timer;
    bool   m_IsIdle;

    /**
     * Shared parameters block.
     */
    sWorkerParameters m_Params;

    /**
     * Global status description.
     */
    QHASGlobal::eStatus m_GlobalStatus;

public:
     Worker (void);
    ~Worker (void);

    IWork * addLayerPreview (IConnector    * connector,
                             ICanvas       * canvas,
                             ICanvasLayer  * layer,
                             IOpenGLHelper * helper);

    IWork * addCanvasUpdate (IConnector    * connector,
                             ICanvas       * canvas,
                             IOpenGLHelper * helper);

    /**
     * Start the timer controlling
     * the tasks execution.
     */
    void start (void);

    /**
     * Stop all works and detach
     * all attached objects.
     */
    void terminate (void);

    void setPaintingStyle (IPaintingStyle * paint) {
        m_Params.paint = paint;
    }

signals:

    /**
     * Indicate that the work status
     * has been changed.
     */
    void statusChanged (QHASGlobal::eStatus status);

public slots:
    void userStartedPainting (void);
    void userStoppedPainting (void);
    void paperChanged        (sPaper*);

private slots:
    void itistime (void);
};

//#endif // QT_OPENGL_LIB
#endif // WORKER_H
