/* Qaquarelle
 * Copyright (C) 2009 Vasiliy Makarov <drmoriarty.0@gmail.com>
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "preferences.h"

Preferences::Preferences()
{
    // [castedflake] Set box-style color picker as default.
    // [castedflake] Set canvas scaling by mouse-scrolling as default.

    QSettings set;
    smartStroke = set.value("smartStroke", false).toBool();
    colorPickerStyle = set.value("colorPickerStyle", 1).toInt();
    //wheelAction = (WheelAction)set.value("wheelAction", WheelScalesCanvas).toInt();
    settingsVersion = set.value("settingsVersion", 0).toInt();

    bool keysLoaded = false;
    for(int i=0; i<DRAGSTATE_COUNT; i++) {
        dragKeys[i] = set.value(QString("dragKey_%1").arg(QString::number(i)), -1).toInt();
        keysLoaded |= dragKeys[i] != -1;
    }
    for(int i=0; i<WheelCOUNT; i++) {
        wheelKeys[i] = set.value(QString("wheelKey_%1").arg(QString::number(i)), -1).toInt();
        keysLoaded |= wheelKeys[i] != -1;
    }

    pressureControl = set.value("pressureControl", false).toBool();
    pressureMin = set.value("pressureMin", .0f).toDouble();
    pressureMax = set.value("pressureMax", 1.f).toDouble();

    // [castedflake] If there are no keys defined and
    // there were no key defined ever, then we set the
    // default ones.
    bool keysDefined = set.value("keysDefined", false).toBool();
    if (!keysDefined && !keysLoaded) {
        dragKeys[DRAGSTATE_CANVAS_MOVE]  = Qt::Key_Space;
        dragKeys[DRAGSTATE_CANVAS_SCALE] = Qt::ControlModifier;
        wheelKeys[WheelScalesCanvas]     = 0;
    }

    // Rendering options.
    render_paintBump = set.value("render_paintBump", true).toBool();
    render_paperBump = set.value("render_paperBump", true).toBool();
    render_mvectors  = set.value("render_mvectors", false).toBool();

    // OpenGL options.
    ogl_textureCompression = set.value ("ogl_textureCompression", 0).toInt();
	strokeQuality = set.value("strokeQuality", 3).toInt();
}

void Preferences::save()
{
	QSettings set;
	set.setValue("smartStroke", smartStroke);
	set.setValue("colorPickerStyle", colorPickerStyle);
	//set.setValue("wheelAction", (int)wheelAction);
	set.setValue("settingsVersion", settingsVersion);
	for(int i=0; i<DRAGSTATE_COUNT; i++) {
		set.setValue(QString("dragKey_%1").arg(QString::number(i)), dragKeys[i]);
	}
	for(int i=0; i<WheelCOUNT; i++) {
		set.setValue(QString("wheelKey_%1").arg(QString::number(i)), wheelKeys[i]);
	}
	set.setValue("pressureControl", pressureControl);
	set.setValue("pressureMin", pressureMin);
	set.setValue("pressureMax", pressureMax);
    set.setValue("keysDefined", true);

    // Rendering options.
    set.setValue("render_paintBump", render_paintBump);
    set.setValue("render_paperBump", render_paperBump);
    set.setValue("render_mvectors",  render_mvectors);

    // OpenGL options.
    set.setValue ("ogl_textureCompression", ogl_textureCompression);
	set.setValue("strokeQuality", strokeQuality);
}

Preferences& Preferences::Self()
{
	static Preferences instance;
	return instance;
}
