/***************************************************************************
 *  Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "channel.h"
#include "commmath.h"
#include <math.h>       // FIMXE Clean up.
#include <stdlib.h>     // FIMXE Clean up.


static inline float fract (float x) {
    return x - floorf (x);
}

/// Factory call.
IChannel * g_fac_SPAWN_CHANNEL (FOURCC uid, int width, int height, IOpenGLHelper * helper) {
    return new Channel_gl (uid, width, height, helper);
}
/*
static int g_TOUCH_COMPARATOR (const void * p0, const void * p1) {
    const sTouchData * touch0 = (const sTouchData *) p0;
    const sTouchData * touch1 = (const sTouchData *) p1;

    return (touch0->patch->cellX () + touch0->patch->cellY () * 128) -
           (touch1->patch->cellX () + touch1->patch->cellY () * 128);
}
*/
const char * IChannel::nameFromUID (FOURCC uid) {
    if (uid == CHANNELUID_Optical) {
        return "Paints::Optical";
    } else
    if (uid == CHANNELUID_Meta0) {
        return "Paints::Meta0";
    } else {
        return NULL;
    }
}

Channel_gl::Channel_gl (FOURCC uid, int width, int height, IOpenGLHelper * helper) {
    m_Id        = ISpawnedObject::allocateId ();
    m_Uid       = uid;
    m_Data      = NULL;
    m_Helper    = helper;
    m_Changed   = FALSE;
    m_GLinited  = FALSE;

   ISpawnedObject::_register (this);

   if (uid == CHANNELUID_Optical) {
       m_StackPosition = 0;
   } else
   if (uid == CHANNELUID_Meta0) {
       m_StackPosition = 1;
   } else {
       m_StackPosition = -1;
   }

   m_LayerDpi        = 100;
   m_LastPatchSize_x = 0;
   m_LastPatchSize_y = 0;

   if (m_Helper) {
       m_Helper->addRef ();

       // Allocate a channel buffer as RGBA texture.
       m_Data = g_fac_SPAWN_ARRAYGRID (m_Helper, width, height,
                                       m_Helper->proposeTextureFormat (OPENGLHELPER_TEXTURE_RGBA));
   } else {
       m_Data = NULL;
   }

   m_glslCopy     = NULL;
   m_glslTouch    = NULL;
   m_glslFeedback = NULL;
}

Channel_gl::~Channel_gl (void) {
    ISpawnedObject::_unregister (this);

    SAFE_RELEASE (m_glslCopy);
    SAFE_RELEASE (m_glslTouch);
    SAFE_RELEASE (m_glslFeedback);
    SAFE_RELEASE (m_Data);
    SAFE_RELEASE (m_Helper);
}

void Channel_gl::checkInited (void) {
    if (m_Helper && !m_GLinited) {
        reloadShaders ();
        m_GLinited = TRUE;
   }
}

void Channel_gl::reloadShaders (void) {
    IOpenGLShader * sh = m_Helper->loadShader ("copy.glsl");
    if (sh) {
        SAFE_RELEASE (m_glslCopy);
        m_glslCopy = sh;
    }

    if (m_Uid == CHANNELUID_Meta0)
        sh = m_Helper->loadShader ("brushtouch_u.glsl");
    else {
        sh = m_Helper->loadShader ("brushfeedback.glsl");
        if (sh) {
            SAFE_RELEASE (m_glslFeedback);
            m_glslFeedback = sh;
        }

        sh = m_Helper->loadShader ("brushtouch.glsl");
    }

    if (sh) {
        SAFE_RELEASE (m_glslTouch);
        m_glslTouch = sh;
    }

    if (m_glslCopy) {
        m_glPar0_src = m_glslCopy->getUniformLocation ("src");
    }
    if (m_glslFeedback) {
        m_glPar2_mask         = m_glslFeedback->getUniformLocation ("mask");
        m_glPar2_patch        = m_glslFeedback->getUniformLocation ("patch");
        m_glPar2_color        = m_glslFeedback->getUniformLocation ("color");
        m_glPar2_color_txmu   = m_glslFeedback->getUniformLocation ("color_txmu");
        m_glPar2_color_txadd  = m_glslFeedback->getUniformLocation ("color_txadd");
        m_glPar2_suckrate     = m_glslFeedback->getUniformLocation ("suckrate");
        m_glPar2_exhaustrate  = m_glslFeedback->getUniformLocation ("exhaustrate");
    }
    if (m_glslTouch) {
        m_glPar1_primitive       = m_glslTouch->getUniformLocation ("primitive");
        m_glPar1_paper           = m_glslTouch->getUniformLocation ("paper");
        m_glPar1_oldpatch        = m_glslTouch->getUniformLocation ("oldpatch");
        m_glPar1_paper_txmu      = m_glslTouch->getUniformLocation ("paper_txmu");
        m_glPar1_paper_txadd     = m_glslTouch->getUniformLocation ("paper_txadd");
        if (m_Uid == CHANNELUID_Meta0) {
            m_glPar1_brush_uv    = m_glslTouch->getUniformLocation ("brush_uv");
            m_glPar1_control     = m_glslTouch->getUniformLocation ("control");
            m_glPar1_controlmask = UINT_UNDEFINED;
            m_glPar1_color       = UINT_UNDEFINED;
            m_glPar1_patch_txmu  = UINT_UNDEFINED;
            m_glPar1_patch_txadd = UINT_UNDEFINED;
            m_glPar1_color_txmu  = UINT_UNDEFINED;
            m_glPar1_color_txadd = UINT_UNDEFINED;
            m_glPar1_world_txmu  = UINT_UNDEFINED;
            m_glPar1_world_txadd = UINT_UNDEFINED;
            m_glPar1_bleedrate   = UINT_UNDEFINED;
            m_glPar1_suckrate    = UINT_UNDEFINED;
            m_glPar1_thickness   = UINT_UNDEFINED;
            m_glPar1_sensitivity = UINT_UNDEFINED;
            m_glPar1_noise_freq  = UINT_UNDEFINED;
            m_glPar1_noise_amp   = UINT_UNDEFINED;
            m_glPar1_noise_re    = UINT_UNDEFINED;
            m_glPar1_noise_im    = UINT_UNDEFINED;
        } else {
            m_glPar1_brush_uv    = UINT_UNDEFINED;
            m_glPar1_control     = UINT_UNDEFINED;
            m_glPar1_controlmask = m_glslTouch->getUniformLocation ("ctl_mask");
            m_glPar1_color       = m_glslTouch->getUniformLocation ("color");
            m_glPar1_patch_txmu  = m_glslTouch->getUniformLocation ("patch_txmu");
            m_glPar1_patch_txadd = m_glslTouch->getUniformLocation ("patch_txadd");
            m_glPar1_color_txmu  = m_glslTouch->getUniformLocation ("color_txmu");
            m_glPar1_color_txadd = m_glslTouch->getUniformLocation ("color_txadd");
            m_glPar1_world_txmu  = m_glslTouch->getUniformLocation ("world_txmu");
            m_glPar1_world_txadd = m_glslTouch->getUniformLocation ("world_txadd");
            m_glPar1_bleedrate   = m_glslTouch->getUniformLocation ("bleedrate");
            m_glPar1_suckrate    = m_glslTouch->getUniformLocation ("suckrate");
            m_glPar1_thickness   = m_glslTouch->getUniformLocation ("thickness");
            m_glPar1_sensitivity = m_glslTouch->getUniformLocation ("sensitivity");
            m_glPar1_noise_freq  = m_glslTouch->getUniformLocation ("noise_freq");
            m_glPar1_noise_amp   = m_glslTouch->getUniformLocation ("noise_amp");
            m_glPar1_noise_re    = m_glslTouch->getUniformLocation ("noise_re");
            m_glPar1_noise_im    = m_glslTouch->getUniformLocation ("noise_im");
        }
    }
}

void Channel_gl::beginGl (void) {
    // Setup the modelview matrix.
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix ();
    glLoadIdentity ();
    glTranslatef (0.0f,  0.0f, -1.0f);

    // Setup the projection matrix.
    glMatrixMode (GL_PROJECTION);
    glPushMatrix ();
    glLoadIdentity ();
    glOrtho (-0.5f, +0.5f, +0.5f, -0.5f, +0.0f, +1.0f);
    glScalef (1.0f, -1.0f, 1.0f);
}

void Channel_gl::endGl (void) {
    glMatrixMode (GL_PROJECTION);
    glPopMatrix ();

    glMatrixMode(GL_MODELVIEW);
    glPopMatrix ();
}

void Channel_gl::resize (int width, int height) {
    // FIXME Implement the channel resizing!
    // m_Data->resize (width, height);
    m_Changed = TRUE;

    checkInited ();
}

void Channel_gl::touch (IInstrumentLockEx * operation,
                        const sTouch * stroke,
                        int length) {
    PROFILE_FUNC();

    QMutexLocker locker (m_Helper->mutex ());
    checkInited ();

    if (NULL == m_glslCopy  ||
        NULL == m_glslTouch ||
        NULL == m_Helper    ||
        NULL == operation->primitiveColor ())
        return;

    if (lock ()) {
        beginGl ();

        // Allocate a temporary patch storage.
        GLuint patchBk  = m_Helper->sharedPatchLock ();
        GLuint feedback = operation->primitiveColor ()->lock_gltex ();

        // Init the patch rendering.
        IFBOWrapper * fbo0 = m_Helper->sharedFBO (OPENGLHELPER_FBO_PATCH);
        if (!fbo0) {
            endGl ();
            unlock ();
            return;
        }

        GLenum patchBk_attachment  = UINT_UNDEFINED;
        GLenum feedback_attachment = UINT_UNDEFINED;
        GLuint glNoise_re          = UINT_UNDEFINED;
        GLuint glNoise_im          = UINT_UNDEFINED;

        fbo0->bind ();
        if (m_Uid == CHANNELUID_Optical) {
            m_Helper->bindTexture_ex (patchBk,  3);

            patchBk_attachment  = fbo0->attachTexture (patchBk,  1);
            feedback_attachment = fbo0->attachTexture (feedback, 1);
        }

        // Noise textures.
        IArray * noise_re = m_Helper->sharedTexture (OPENGLHELPER_TEX_NOISE0);
        IArray * noise_im = m_Helper->sharedTexture (OPENGLHELPER_TEX_NOISE1);
        if (noise_re && noise_im && m_Uid == CHANNELUID_Optical) {
            glNoise_re = noise_re->lock_gltex ();
            glNoise_im = noise_im->lock_gltex ();
        }

        const float patch_dx = 0.5f * operation->scalingFactor () * operation->primitiveShape ()->width ();
        const float patch_dy = 0.5f * operation->scalingFactor () * operation->primitiveShape ()->height ();

      //const float patch_bk_tx0 = 0.0f;
      //const float patch_bk_ty0 = 0.0f;
        const float patch_bk_tx1 = 2.0f * patch_dx / m_Helper->sharedPatchSize ();
        const float patch_bk_ty1 = 2.0f * patch_dy / m_Helper->sharedPatchSize ();

        const float color_tmu = ((float)INSTRUMENT_COLORBUFFER_SIZE) /
                                        m_Helper->sharedPatchSize ();

        // Setup the shader.
        m_glslTouch->useProgram ();
        m_glslTouch->uniform1i  (m_glPar1_primitive,   0);
        m_glslTouch->uniform1i  (m_glPar1_paper,       2);
        m_glslTouch->uniform1i  (m_glPar1_oldpatch,    3);
        m_glslTouch->uniform4f  (m_glPar1_controlmask,
                                 operation->flowControlComponentMask ()[0],
                                 operation->flowControlComponentMask ()[1],
                                 operation->flowControlComponentMask ()[2],
                                 operation->flowControlComponentMask ()[3]);
        if (m_Uid == CHANNELUID_Meta0) {
            m_glslTouch->uniform2f  (m_glPar1_brush_uv,
                                     operation->velocity ()[0],
                                     operation->velocity ()[1]);
        } else {
            m_glslTouch->uniform1i  (m_glPar1_color,       1);
            m_glslTouch->uniform1i  (m_glPar1_noise_re,    4);
            m_glslTouch->uniform1i  (m_glPar1_noise_im,    5);
            m_glslTouch->uniform1f  (m_glPar1_sensitivity, operation->stipplingSensitivity ());
            m_glslTouch->uniform1f  (m_glPar1_noise_freq,  operation->noiseParameters ()->frequency);
            m_glslTouch->uniform1f  (m_glPar1_noise_amp,   patch_dx < 4.0f ? 0.1f :
                                                           operation->noiseParameters ()->amplitude);
            m_glslTouch->uniform1f  (m_glPar1_bleedrate,   operation->bleedingRate ());
            m_glslTouch->uniform1f  (m_glPar1_suckrate,    operation->suckingRate ());

            // Thickness of a paint layer depends on paint viscosity.
            m_glslTouch->uniform1f  (m_glPar1_thickness,   0.5f + 0.5f * operation->paintingStyle ()->paintViscosity ());
            m_glslTouch->uniform2f  (m_glPar1_patch_txmu,  patch_bk_tx1, patch_bk_ty1);
            m_glslTouch->uniform2f  (m_glPar1_color_txmu,  color_tmu, color_tmu);
            m_glslTouch->uniform2f  (m_glPar1_world_txmu,
                                     ((float) m_Helper->sharedPatchSize ()) / m_Data->width  (),
                                     ((float) m_Helper->sharedPatchSize ()) / m_Data->height ());
            m_glslTouch->uniform2f  (m_glPar1_world_txadd, 0.0f, 0.0f);

            glActiveTexture (GL_TEXTURE0 + 4);
            glBindTexture   (GL_TEXTURE_2D, glNoise_re);
            glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
            glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
            glTexParameteri (GL_TEXTURE_2D, GL_GENERATE_MIPMAP,    GL_FALSE);
            glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     GL_REPEAT);
            glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     GL_REPEAT);

            glActiveTexture (GL_TEXTURE0 + 5);
            glBindTexture   (GL_TEXTURE_2D, glNoise_im);
            glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
            glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
            glTexParameteri (GL_TEXTURE_2D, GL_GENERATE_MIPMAP,    GL_FALSE);
            glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     GL_REPEAT);
            glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     GL_REPEAT);
        }

        sTouchData TOUCHES[1024];
        int        NTOUCHES = 0;

        // Buffer the touches.
        for (int i = 0; i < length; i++) {
            const int x = stroke[i].x;
            const int y = stroke[i].y;

            // Get the list of all patches that are to be touched.
            // (real patch size is twice smaller that the patch dimentions).
            IArrayGridPatch * patches[32];
            const int nPatches = m_Data->touched (
                (int)(x - patch_dx),
                (int)(y - patch_dy),
                (int)(2 * patch_dx),
                (int)(2 * patch_dy),
                patches
            );

            for (int j = 0; j < nPatches; j++) {
                if (!patches[j])
                    continue;

                TOUCHES[NTOUCHES].x     = x;
                TOUCHES[NTOUCHES].y     = y;
                TOUCHES[NTOUCHES].feedback = stroke[i].feedback;
                TOUCHES[NTOUCHES].patch    = patches[j];
                NTOUCHES++;
            }
        }

        // Sort the touches by patch to reduce the
        // swapping of render targets.
        //qsort (TOUCHES, NTOUCHES, sizeof(sTouchData), g_TOUCH_COMPARATOR);

        // Process all touches.
        for (int i = 0; i < NTOUCHES; i++) {
            const int x = TOUCHES[i].x;
            const int y = TOUCHES[i].y;

            // Setup the paper texture parameters.
            const float paper_x = fract ((float)(x - patch_dx) / operation->paperTexture ()->width ());
            const float paper_y = fract ((float)(y - patch_dy) / operation->paperTexture ()->height ());

            float paper_scale_x = 2.0f * patch_dx / operation->paperTexture ()->width ();
            float paper_scale_y = 2.0f * patch_dy / operation->paperTexture ()->height ();

            // Scale the paper texture according to its dpi value.
            paper_scale_x *= (float) m_LayerDpi / operation->paperDpi ();
            paper_scale_y *= (float) m_LayerDpi / operation->paperDpi ();

            m_glslTouch->useProgram ();
            m_glslTouch->uniform2f  (m_glPar1_paper_txadd, paper_x, 1.0f - paper_y);
            m_glslTouch->uniform2f  (m_glPar1_paper_txmu,  paper_scale_x, -paper_scale_y);

            IArray * pData = TOUCHES[i].patch->data ();
            if (!pData)
                continue;

            // Try to add the patch to transaction.
            if (operation->transaction ()) {
                operation->transaction ()->addImagePatch (
                    pData, m_Uid,
                    TOUCHES[i].patch->cellX (),
                    TOUCHES[i].patch->cellY ()
                );
            }

            // Mark the cell as modified.
            TOUCHES[i].patch->touched ();

            // Setup the touch boundaries in patch coords.
            const int patch_left = TOUCHES[i].patch->rect ()->left;
            const int patch_top  = TOUCHES[i].patch->rect ()->top;
            const float left_u   = 2.0f * (x - patch_dx - patch_left) / m_Helper->sharedPatchSize ();
            const float right_u  = 2.0f * (x + patch_dx - patch_left) / m_Helper->sharedPatchSize ();
            const float top_u    = 2.0f * (y - patch_dy - patch_top)  / m_Helper->sharedPatchSize ();
            const float bottom_u = 2.0f * (y + patch_dy - patch_top)  / m_Helper->sharedPatchSize ();
            const float left   = -0.5f + left_u;
            const float right  = -0.5f + right_u;
            const float top    = -0.5f + top_u;
            const float bottom = -0.5f + bottom_u;

            const float world_left = 1.0f * ((float) x - patch_dx) / m_Data->width ();
            const float world_top  = 1.0f * ((float) y - patch_dy) / m_Data->height ();

            GLuint   glSource  = UINT_UNDEFINED;
            IArray * primitive = operation->primitiveShape ();
/*
            unsigned long key = g_MAKE_FOURCC (TOUCHES[i].patch->cellX (),
                                               TOUCHES[i].patch->cellY (),
                                               stackPosition (),
                                               1);

            // Check if we have a backup patch in cache.
            if (m_Helper->sharedPatchCache ()->contains (key)) {
                patch_cache = m_Helper->sharedPatchCache ()->value (key);
            }
*/
            // Generate an instrument feedback if needed.
            if (NULL != m_glslFeedback &&
                NULL != primitive &&
                TOUCHES[i].feedback &&
                m_Uid == CHANNELUID_Optical &&
                IS_SIGNIFICANT (operation->suckingRate ())) {

                // Backup the color patch.
                m_glslCopy->useProgram ();
                m_glslCopy->uniform1i (m_glPar0_src, 1);
                m_Helper->bindTexture_ex (feedback,  1);

                // Render (copy as rendering).
                glDrawBuffer (patchBk_attachment);
                glViewport (0, 0, INSTRUMENT_COLORBUFFER_SIZE * 2,
                                  INSTRUMENT_COLORBUFFER_SIZE);

                //const float margin = 0.00f;
                glBegin (GL_QUADS);
                    glMultiTexCoord2f (GL_TEXTURE0, 0.0f, 0.0f);
                    glVertex3f (-0.5f, -0.5f, 1.0f);

                    glMultiTexCoord2f (GL_TEXTURE0, 2.0f * color_tmu, 0.0f);
                    glVertex3f (+0.5f, -0.5f, 1.0f);

                    glMultiTexCoord2f (GL_TEXTURE0, 2.0f * color_tmu, color_tmu);
                    glVertex3f (+0.5f, +0.5f, 1.0f);

                    glMultiTexCoord2f (GL_TEXTURE0, 0.0f, color_tmu);
                    glVertex3f (-0.5f, +0.5f, 1.0f);
                glEnd();

                glSource = pData->lock_gltex ();

                m_Helper->bindTexture_ex (glSource,    1, GL_LINEAR);
                m_Helper->bindTexture_ex (patchBk,     2);

                // Number of steps skipped.
                const int steps = MAX (1, (int)(0.05f / operation->suckingRate ()));

                m_glslFeedback->useProgram ();
                m_glslFeedback->uniform1i  (m_glPar2_patch,       1);
                m_glslFeedback->uniform1i  (m_glPar2_color,       2);
                m_glslFeedback->uniform2f  (m_glPar2_color_txmu,  color_tmu, color_tmu);
                m_glslFeedback->uniform1f  (m_glPar2_exhaustrate, 0.05f +
                                            operation->suckingRate () *
                                            operation->paintingStyle ()->paintViscosity () * steps);
                m_glslFeedback->uniform1f  (m_glPar2_suckrate, MIN (
                                            1.0f, operation->suckingRate () * steps));

              //glEnable     (GL_SCISSOR_TEST);
                glDrawBuffer (feedback_attachment);

                float plane_x = 0.0f;
                float plane_y = 0.0f;
                if (operation->affectsDataPlanes () & DATAPLANE_DRY) {
                    plane_y += 0.5f;
                }

                for (int x = 0; x < 2; x++) {
                    m_glslFeedback->uniform2f (m_glPar2_color_txadd, x * color_tmu, 0.0f);

                    // We got to check the out-of-subpatch leaks.
                    const float left_clamp   = 0.01f - (MIN (left_u,   0.01f));
                    const float top_clamp    = 0.01f - (MIN (top_u,    0.01f));
                    const float right_clamp  = 1.99f - (MAX (right_u,  0.99f));
                    const float bottom_clamp = 1.99f - (MAX (bottom_u, 0.99f));

                    const int win_x = INSTRUMENT_COLORBUFFER_SIZE * x;
                    const int win_y = (operation->affectsDataPlanes () & DATAPLANE_WET) ? 0 : INSTRUMENT_COLORBUFFER_SIZE;

                    glViewport (win_x, win_y, INSTRUMENT_COLORBUFFER_SIZE, INSTRUMENT_COLORBUFFER_SIZE);
                  //glScissor  (wl, wt, ww, wh);

                    // Render.
                    glBegin (GL_QUADS);
                        glMultiTexCoord2f (GL_TEXTURE0, plane_x + .5 * MAX (0.01f, left_u),
                                                        plane_y + .5 * MAX (0.01f, top_u));
                        glMultiTexCoord2f (GL_TEXTURE1, left_clamp, top_clamp);
                        glVertex3f (-0.5f + left_clamp, -0.5f + top_clamp, 1.0f);

                        glMultiTexCoord2f (GL_TEXTURE0, plane_x + .5f * MIN (0.99f, right_u),
                                                        plane_y + .5f * MAX (0.01f, top_u));
                        glMultiTexCoord2f (GL_TEXTURE1, right_clamp, top_clamp);
                        glVertex3f (-0.5f + right_clamp, -0.5f + top_clamp, 1.0f);

                        glMultiTexCoord2f (GL_TEXTURE0, plane_x + .5f * MIN (0.99f, right_u),
                                                        plane_y + .5f * MIN (0.99f, bottom_u));
                        glMultiTexCoord2f (GL_TEXTURE1, right_clamp, bottom_clamp);
                        glVertex3f (-0.5f + right_clamp, -0.5f + bottom_clamp, 1.0f);

                        glMultiTexCoord2f (GL_TEXTURE0, plane_x + .5f * MAX (0.01f, left_u),
                                                        plane_y + .5f * MIN (0.99f, bottom_u));
                        glMultiTexCoord2f (GL_TEXTURE1, left_clamp, bottom_clamp);
                        glVertex3f (-0.5f + left_clamp, -0.5f + bottom_clamp, 1.0f);
                    glEnd ();

                    plane_x += 0.5f;
                }

                glDisable (GL_SCISSOR_TEST);

                pData->unlock ();
                primitive->unlock ();
            }

            // Backup the whole patch.
            if (m_Uid == CHANNELUID_Optical) {
                glSource = pData->lock_gltex ();

                m_glslCopy->useProgram ();
                m_glslCopy->uniform1i (m_glPar0_src, 0);
                m_Helper->bindTexture_ex (glSource,  0);

                glDrawBuffer (patchBk_attachment);
                glViewport (0, 0, m_Helper->sharedPatchSize (), m_Helper->sharedPatchSize ());

                // Render (copy as rendering).
                glCallList (m_Helper->sharedList (OPENGLHELPER_LIST_QUAD1));

                // Unlock the data surface.
                pData->unlock ();
            }

            // Render the primitive now.
            if (primitive) {
                static const int g_PackedVertexOffset    = sizeof(float) * 0;
                static const int g_PackedTexCoord0Offset = sizeof(float) * 3;
                static const int g_PackedTexCoord1Offset = sizeof(float) * 5;
                static const int g_PackedFullSize        = sizeof(float) * 7;

                // Lock the required data.
                GLuint glPrimitive = primitive->lock_gltex ();
                GLuint glTarget    = pData->lock_gltex ();
                GLuint glPaper     = operation->paperTexture ()->lock_gltex ();

                m_Helper->bindTexture_ex (glPrimitive, 0);
                if (m_Uid == CHANNELUID_Optical) {
                    m_Helper->bindTexture_ex (feedback,    1, GL_LINEAR);
                    m_Helper->bindTexture_ex (glPaper,     2);
                    m_Helper->bindTexture_ex (patchBk,     3);
                }

                GLenum target_attachment = fbo0->attachTexture (glTarget);

                glDrawBuffer (target_attachment);
                glViewport (0, 0, m_Helper->sharedPatchSize (), m_Helper->sharedPatchSize ());

                int nVertices = 0;
                glBindBufferARB      (GL_ARRAY_BUFFER_ARB, operation->primitiveGeometry (nVertices));

                glEnableClientState  (GL_VERTEX_ARRAY);
                glVertexPointer      (3, GL_FLOAT, g_PackedFullSize, BUFFER_OFFSET(g_PackedVertexOffset));

                glClientActiveTextureARB (GL_TEXTURE0_ARB);
                glEnableClientState      (GL_TEXTURE_COORD_ARRAY);
                glTexCoordPointer        (2, GL_FLOAT, g_PackedFullSize, BUFFER_OFFSET(g_PackedTexCoord0Offset));

                glClientActiveTextureARB (GL_TEXTURE1_ARB);
                glEnableClientState      (GL_TEXTURE_COORD_ARRAY);
                glTexCoordPointer        (2, GL_FLOAT, g_PackedFullSize, BUFFER_OFFSET(g_PackedTexCoord1Offset));

                // Clamp.
                nVertices = MIN (nVertices, 128);

                // Prepare the indices.
                BYTE vindices[128];
                for (int i = 0; i < nVertices; vindices[i] = i, i++) ;
                vindices[nVertices] = 1;

                m_glslTouch->useProgram ();

                glEnable     (GL_SCISSOR_TEST);
                glMatrixMode (GL_MODELVIEW);
                glPushMatrix ();

                const int off    = m_Helper->sharedPatchSize () / 2;
                const int planes = m_Uid == CHANNELUID_Optical ? 2 : 1;
                for (int plane = 0; plane < planes; plane++) {
                    // Check if the instrument affects this data plane.
                    if (m_Uid == CHANNELUID_Optical) {
                        if ((plane == 0 && !(operation->affectsDataPlanes () & DATAPLANE_WET)) ||
                            (plane == 1 && !(operation->affectsDataPlanes () & DATAPLANE_DRY)))
                            continue;
                    }

                    const float plane_y  = plane == 0 ? -0.25f : +0.25f;
                    const float plane_yt = plane == 0 ?  0.00f :  0.50f;

                    // Process sub-channels.
                    for (int sub = 0; sub < planes; sub++) {
                        const float plane_x  = sub == 0 ? -0.25f : +0.25f;
                        const float plane_xt = sub == 0 ?  0.00f :  0.50f;

                        // Set the transform for patch geometry.
                        glLoadIdentity ();
                        glTranslatef (
                               plane_x + 0.25f * (left + right),
                               plane_y + 0.25f * (bottom + top),
                               -1.0f
                        );
                        glScalef  (0.5f * (right - left), 0.5f * (bottom - top), 1.0f);
                        glRotatef (operation->primitiveRotation (), 0.0f, 0.0f, 1.0f);
                        glScissor (sub   == 0 ? 0 : off,
                                   plane == 0 ? 0 : off,
                                   off, off
                        );

                        if (m_Uid == CHANNELUID_Optical) {
                            m_glslTouch->uniform2f (m_glPar1_color_txadd,
                                                    sub == 0 ? 0.0f : color_tmu, 0.0f);
                            m_glslTouch->uniform2f (m_glPar1_patch_txadd,
                                                    plane_xt + 0.5f * left_u,
                                                    plane_yt + 0.5f * top_u);
                            m_glslTouch->uniform2f (m_glPar1_world_txadd,
                                                    world_left + left_u,
                                                    world_top  + top_u);
                        }

                        // Render the brush as a triangle fan.
                        glDrawElements (GL_TRIANGLE_FAN, nVertices + 1, GL_UNSIGNED_BYTE, vindices);
                    }
                }

                glDisable                (GL_SCISSOR_TEST);
                glDisableClientState     (GL_VERTEX_ARRAY);
                glClientActiveTextureARB (GL_TEXTURE0_ARB);
                glDisableClientState     (GL_TEXTURE_COORD_ARRAY);
                glClientActiveTextureARB (GL_TEXTURE1_ARB);
                glDisableClientState     (GL_TEXTURE_COORD_ARRAY);
                glBindBufferARB          (GL_ARRAY_BUFFER_ARB, 0);

                glMatrixMode (GL_MODELVIEW);
                glPopMatrix ();

                // Unlock all data.
                pData->unlock ();
                primitive->unlock ();
                operation->paperTexture ()->unlock ();
            }
        }

        fbo0->attachTexture (patchBk,  0);
        fbo0->attachTexture (feedback, 0);

        operation->primitiveColor ()->unlock ();
        if (noise_re) {
            noise_re->unlock ();
        }
        if (noise_im) {
            noise_im->unlock ();
        }

        // Unlock the data used.
        m_Helper->sharedPatchUnlock (patchBk);

#ifdef DEBUG_GLFINISH
        glFinish ();
#endif

        endGl ();
        unlock ();
        m_Changed = TRUE;
    }
}

void Channel_gl::clear (void) {
    if (m_Data) {
        m_Data->zero ();
        m_Changed = TRUE;
    }
}

unsigned long Channel_gl::requiredBufferSize (void) const {
    if (m_Data) {
        return m_Data->element_size () * m_Data->width () * m_Data->height () +
               3 * sizeof(qint32);
    }

    return 0;
}

#include <QImage>
unsigned long Channel_gl::saveData (QDataStream * stream, unsigned long capacity) {
    checkInited ();

    unsigned long written = 0L;
    if (m_Data) {
        stream->operator << ((qint32)  m_Data->width ());
        stream->operator << ((qint32)  m_Data->height ());
        stream->operator << ((quint32) m_Uid);

        written += 2 * sizeof(qint32) + 1 * sizeof(quint32);
        for (int y = 0; y < m_Data->nPatchesY (); y++) {
            for (int x = 0; x < m_Data->nPatchesX (); x++) {
                IArray * patch = m_Data->patch (x, y)->data ();
                if (patch) {
                    unsigned long toCopy = patch->element_size () * patch->width () * patch->height ();
                                  toCopy = MIN (toCopy, (unsigned long)capacity);

                    const char * p = (const char *) patch->lock_pointer_const ();
                    //const char * srcl = p;
                    if (p) {
                        for (unsigned long wr = 0; wr < toCopy; wr += stream->writeRawData (p + wr, toCopy - wr)) ;
/*
                        // FIXME REMOVE!!
                        if (x == 0 &&
                            y == 0 &&
                            m_Uid == CHANNELUID_Meta0) {
                            QImage i (patch->width (), patch->height (), QImage::Format_ARGB32);

                            for (int y = 0; y < patch->height (); y++) {
                                QRgb * line = (QRgb*) i.scanLine (y);
                                const BYTE * src = (const BYTE*) srcl;
                                for (int x = 0; x < patch->width (); x++) {
                                    line[x] = qRgb (src[0], src[1], src[2]);
                                    src += 4;
                                }

                                srcl += patch->width () * 4;
                            }

                            i.save ("patchdump.png");
                        }
*/
                        patch->unlock ();
                        written += toCopy;
                    }
                }
            }
        }
    }

    return written;
}

unsigned long Channel_gl::loadData (QDataStream * stream, unsigned long capacity) {
    unsigned long read = 0L;
    if (m_Data) {
        for (int y = 0; y < m_Data->nPatchesY (); y++) {
            for (int x = 0; x < m_Data->nPatchesX (); x++) {
                IArray * patch = m_Data->patch (x, y)->data ();
                if (patch) {
                    unsigned long toCopy = patch->element_size () * patch->width () * patch->height ();
                                  toCopy = MIN (toCopy, (unsigned long)capacity);

                    char * p = (char *) patch->lock_pointer ();
                    if (p) {
                        for (unsigned long r = 0; r < toCopy; r += stream->readRawData (p + r, toCopy - r)) ;
                        patch->unlock ();
                        read += toCopy;
                    }
                }
            }
        }
    }

    return read;
}
